@@ -82,7 +82,7 @@ pub struct Config {
     /// that servers use if required to verify a client certificate
     /// by the policy in client_auth.
     /// Used by Server to verify client's certificate
-    pub client_cert_verifier: Option<Arc<dyn rustls::ClientCertVerifier>>,
+    pub client_cas: rustls::RootCertStore,
 
     /// server_name is used to verify the hostname on the returned
     /// certificates unless insecure_skip_verify is given.
@@ -115,7 +115,7 @@ impl Default for Config {
             insecure_hashes: false,
             verify_peer_certificate: None,
             roots_cas: rustls::RootCertStore::empty(),
-            client_cert_verifier: None,
+            client_cas: rustls::RootCertStore::empty(),
             server_name: String::default(),
             mtu: 0,
             replay_protection_window: 0,
