@@ -10,17 +10,16 @@ use crate::curve::named_curve::*;
 use crate::error::*;
 use crate::record_layer::record_layer_header::*;
 
-use der_parser::{oid, oid::Oid};
-
 use anyhow::Result;
+use der_parser::{oid, oid::Oid};
+use rcgen::KeyPair;
 use ring::rand::SystemRandom;
 use ring::signature::{EcdsaKeyPair, Ed25519KeyPair, RsaKeyPair};
-use sha2::{Digest, Sha256};
 use std::sync::Arc;
 
 #[derive(Clone, PartialEq)]
 pub struct Certificate {
-    pub certificate: rustls::Certificate,
+    pub certificate: Vec<rustls::Certificate>,
     pub private_key: CryptoPrivateKey,
 }
 
@@ -62,7 +61,7 @@ impl Certificate {
         };
 
         Ok(Certificate {
-            certificate: rustls::Certificate(certificate),
+            certificate: vec![rustls::Certificate(certificate)],
             private_key,
         })
     }
@@ -109,7 +108,7 @@ impl Certificate {
         };
 
         Ok(Certificate {
-            certificate: rustls::Certificate(certificate),
+            certificate: vec![rustls::Certificate(certificate)],
             private_key,
         })
     }
@@ -197,6 +196,42 @@ impl Clone for CryptoPrivateKey {
     }
 }
 
+impl CryptoPrivateKey {
+    pub fn from_key_pair(key_pair: &KeyPair) -> Result<Self> {
+        let serialized_der = key_pair.serialize_der();
+        if key_pair.is_compatible(&rcgen::PKCS_ED25519) {
+            Ok(CryptoPrivateKey {
+                kind: CryptoPrivateKeyKind::Ed25519(
+                    Ed25519KeyPair::from_pkcs8(&serialized_der)
+                        .map_err(|e| Error::new(e.to_string()))?,
+                ),
+                serialized_der,
+            })
+        } else if key_pair.is_compatible(&rcgen::PKCS_ECDSA_P256_SHA256) {
+            Ok(CryptoPrivateKey {
+                kind: CryptoPrivateKeyKind::Ecdsa256(
+                    EcdsaKeyPair::from_pkcs8(
+                        &ring::signature::ECDSA_P256_SHA256_ASN1_SIGNING,
+                        &serialized_der,
+                    )
+                    .map_err(|e| Error::new(e.to_string()))?,
+                ),
+                serialized_der,
+            })
+        } else if key_pair.is_compatible(&rcgen::PKCS_RSA_SHA256) {
+            Ok(CryptoPrivateKey {
+                kind: CryptoPrivateKeyKind::Rsa256(
+                    RsaKeyPair::from_pkcs8(&serialized_der)
+                        .map_err(|e| Error::new(e.to_string()))?,
+                ),
+                serialized_der,
+            })
+        } else {
+            Err(Error::new("Unsupported key_pair".to_owned()).into())
+        }
+    }
+}
+
 // If the client provided a "signature_algorithms" extension, then all
 // certificates provided by the server MUST be signed by a
 // hash/signature algorithm pair that appears in that extension
@@ -241,7 +276,7 @@ pub(crate) fn generate_key_signature(
 pub const OID_ED25519: Oid<'static> = oid!(1.3.101 .112);
 pub const OID_ECDSA: Oid<'static> = oid!(1.2.840 .10045 .2 .1);
 
-pub(crate) fn verify_key_signature(
+fn verify_signature(
     message: &[u8],
     /*_hash_algorithm: HashAlgorithm,*/
     remote_key_signature: &[u8],
@@ -298,6 +333,15 @@ pub(crate) fn verify_key_signature(
     Ok(())
 }
 
+pub(crate) fn verify_key_signature(
+    message: &[u8],
+    /*_hash_algorithm: HashAlgorithm,*/
+    remote_key_signature: &[u8],
+    raw_certificates: &[Vec<u8>],
+) -> Result<()> {
+    verify_signature(message, remote_key_signature, raw_certificates)
+}
+
 // If the server has sent a CertificateRequest message, the client MUST send the Certificate
 // message.  The ClientKeyExchange message is now sent, and the content
 // of that message will depend on the public key algorithm selected
@@ -310,15 +354,11 @@ pub(crate) fn generate_certificate_verify(
     handshake_bodies: &[u8],
     private_key: &CryptoPrivateKey, /*, hashAlgorithm hashAlgorithm*/
 ) -> Result<Vec<u8>> {
-    let mut h = Sha256::new();
-    h.update(handshake_bodies);
-    let hashed = h.finalize();
-
     let signature = match &private_key.kind {
-        CryptoPrivateKeyKind::Ed25519(kp) => kp.sign(hashed.as_slice()).as_ref().to_vec(),
+        CryptoPrivateKeyKind::Ed25519(kp) => kp.sign(handshake_bodies).as_ref().to_vec(),
         CryptoPrivateKeyKind::Ecdsa256(kp) => {
             let system_random = SystemRandom::new();
-            kp.sign(&system_random, hashed.as_slice())
+            kp.sign(&system_random, handshake_bodies)
                 .map_err(|e| Error::new(e.to_string()))?
                 .as_ref()
                 .to_vec()
@@ -329,7 +369,7 @@ pub(crate) fn generate_certificate_verify(
             kp.sign(
                 &ring::signature::RSA_PKCS1_SHA256,
                 &system_random,
-                hashed.as_slice(),
+                handshake_bodies,
                 &mut signature,
             )
             .map_err(|e| Error::new(e.to_string()))?;
@@ -347,11 +387,7 @@ pub(crate) fn verify_certificate_verify(
     remote_key_signature: &[u8],
     raw_certificates: &[Vec<u8>],
 ) -> Result<()> {
-    let mut h = Sha256::new();
-    h.update(handshake_bodies);
-    let hashed = h.finalize();
-
-    verify_key_signature(&hashed, remote_key_signature, raw_certificates)
+    verify_signature(handshake_bodies, remote_key_signature, raw_certificates)
 }
 
 pub(crate) fn load_certs(raw_certificates: &[Vec<u8>]) -> Result<Vec<rustls::Certificate>> {
