@@ -206,7 +206,7 @@ impl Flight for Flight5 {
                     Content::Handshake(Handshake::new(HandshakeMessage::Certificate(
                         HandshakeMessageCertificate {
                             certificate: if let Some(cert) = &certificate {
-                                vec![cert.certificate.as_ref().to_vec()]
+                                cert.certificate.iter().map(|x| x.0.clone()).collect()
                             } else {
                                 vec![]
                             },
